/*--------------------------------------------------
basierend auf den Sourcen und Hinweisen von:

HTTP 1.1 Webserver for ESP8266
for ESP8266 adapted Arduino IDE
Stefan Thesen 04/2015

Running stable for days
(in difference to all samples I tried)

Does HTTP 1.1 with defined connection closing.
Reconnects in case of lost WiFi.
Handles empty requests in a defined manner.
Handle requests for non-exisiting pages correctly.

MIME-Typen: .html, .ico, .css, .js, .jpg, .bin  (teilweise ungetestet)

Dateien im SPIFFS
v0.5 MSoft 25.01.2016
--------------------------------------------------*/

#include <ESP8266WiFi.h>
#include <ESP8266mDNS.h>
#include <WiFiUdp.h>
#include <FS.h>
#include <ArduinoOTA.h>
#include "html_tools.h"

//#define HTML_DEBUG
//#define REPLACE_DEBUG
//#define USER_DEBUG
//#define FILE_DEBUG

#define HOSTNAME "ESP8266-OTA-"                   // Hostename. The setup function adds the Chip ID at the end.

const char* ssid = "xxxxxxxxxx";
const char* password = "xxxxxxxxxxx";

void useraction(void);                            // diese Funktion wird aufgerufen, wenn die Paramter im Array sind bevor die Page gesendet wird
void WiFiStart(void);
void getSensordaten(void);

unsigned long ulReconncount = 0;                  // Anzahl Re-Conncts
unsigned long ulReqcount = 0;                     // Anazhl Zugriffe

// Create an instance of the server on Port 80
WiFiServer server(80);

void setup()
{
  // start serial
  Serial.begin(250000);                            // mit FTDI 250000 möglich 
  delay(1);

  // Initialize file system.
  if (!SPIFFS.begin())
  {
    Serial.println("Failed to mount file system");
    return;
  }

  // Set Hostname.
  String hostname(HOSTNAME);
  hostname += String(ESP.getChipId(), HEX);
  WiFi.hostname(hostname);
  Serial.println("Hostname: " + hostname);
  
  // inital connect
  WiFi.mode(WIFI_STA);
  WiFiStart();

  // Start the server
  server.begin();
  Serial.println("Server gestartet");

  // Start OTA server.
  ArduinoOTA.setHostname((const char *)hostname.c_str());
  ArduinoOTA.begin();
  Serial.println("OTA gestartet");
  Serial.println("");

}

void loop()
{
  // check if WLAN is connected
  if (WiFi.status() != WL_CONNECTED)
  {
    WiFiStart();
  }


  ArduinoOTA.handle();                      // Handle OTA server.

  responseClient();                         // Check if a client has connected 

  getSensordaten();                         // aktualisiert Sensordaten
}

// ----------------------------------------------------------------------
// Dummy-Routine als Beispiel für die Ersetzungen

void getSensordaten()
{
  index_array[0] = "21,4";              // a
  index_array[1] = "42";                // b
  index_array[2] = "- 2,3";             // c
  index_array[3] = "82";                // d
  index_array[4] = "21,8";              // e
  index_array[5] = "41";                // f
  index_array[6] = "16,3";              // g
  index_array[7] = "38";                // h
  index_array[8] = "20,5";              // i
  index_array[9] = "-21,0";             // j

  index_array[10] = "1026";             // k
  
  index_array[11] = "32,0";             // l
  index_array[12] = "0,0";              // m
  index_array[13] = "7,4";              // n
  index_array[14] = "6,5";              // o
  index_array[15] = "94,7";             // p  
}

// ----------------------------------------------------------------------

void useraction()
{ 
  if (pageDaten.Path == "/form.html")
  { 
    index_array[22] = "";                                          // zu bearbeitende Strings löschen, hier vname zname, Zahlmethode, zutat
    index_array[23] = "";
    index_array[24] = "";
  
    for (int i=0; i<MAX_PARAMETER; i++)
    {
#ifdef USER_DEBUG
      Serial.print("Index ");
      Serial.print(i);
      Serial.print(" ");
      Serial.println(pageDaten.Index[i]);
#endif
    
      if (pageDaten.Index[i] == "zutat")
      {
          index_array[23] += " und " + pageDaten.Value[i];                           // x      
      }

      if (pageDaten.Index[i] == "Zahlmethode")
      {
          index_array[24] = "Sie wollen mit " + pageDaten.Value[i] + " bezahlen.";   // y      
      }

      if (pageDaten.Index[i] == "vname")
      {
        index_array[22] += " " + pageDaten.Value[i];                                 // w 
      }
      if (pageDaten.Index[i] == "zname")
      {
        index_array[22] += " " + pageDaten.Value[i];                                 // w 
      }
    }
    index_array[22].trim();                                                          // wo nötig aufräumen und Texte ergänzen
    if (index_array[22] != "")
    {
      index_array[22] = "Sie hei&szlig;en " + index_array[22] + ".<br>";
    }

    if (index_array[23].startsWith(" und "))
    {
      String temp = index_array[23].substring(5);
      index_array[23] = "Sie haben " + temp + " gew&auml;hlt.<br>";
    }

#ifdef USER_DEBUG
    Serial.println("");
    Serial.println("neue Ersetzungen");
    for (int i=0; i<ANZ_REPLACE; i++)
    {
      if (i<10)
      {
        Serial.print(" ");     
      }
      Serial.print(i);
      Serial.print(" - ");
      Serial.print(char('a' + i));   
      Serial.print(" # ");
      Serial.println(index_array[i]);    
    }
    Serial.println("");
#endif

  }  
}

// ----------------------------------------------------------------------

void WiFiStart()
{
  // Connect to WiFi network
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED)
  {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");

  // Print the IP address
  Serial.println(WiFi.localIP());
  randomSeed(millis());
  ulReconncount++;
  index_array[RE_CONNECT] = ulReconncount;              // für die Webseite ins Array eintragen
}


